@@ -4,7 +4,6 @@
 # http://dev.mysql.com/doc/refman/5.5/en/error-messages-client.html
 from __future__ import print_function
 from ._compat import PY2, range_type, text_type, str_type, JYTHON, IRONPYTHON
-DEBUG = False
 
 import errno
 from functools import partial
@@ -17,6 +16,15 @@ import sys
 import traceback
 import warnings
 
+from .charset import MBLENGTH, charset_by_name, charset_by_id
+from .constants import CLIENT, COMMAND, FIELD_TYPE, SERVER_STATUS
+from .converters import (
+    escape_item, encoders, decoders, escape_string, through)
+from .cursors import Cursor
+from .optionfile import Parser
+from .util import byte2int, int2byte
+from . import err
+
 try:
     import ssl
     SSL_ENABLED = True
@@ -24,11 +32,6 @@ except ImportError:
     ssl = None
     SSL_ENABLED = False
 
-if PY2:
-    import ConfigParser as configparser
-else:
-    import configparser
-
 try:
     import getpass
     DEFAULT_USER = getpass.getuser()
@@ -37,13 +40,7 @@ except ImportError:
     DEFAULT_USER = None
 
 
-from .charset import MBLENGTH, charset_by_name, charset_by_id
-from .cursors import Cursor
-from .constants import CLIENT, COMMAND, FIELD_TYPE, SERVER_STATUS
-from .util import byte2int, int2byte
-from .converters import (
-    escape_item, encoders, decoders, escape_string, through)
-from . import err
+DEBUG = False
 
 _py_version = sys.version_info[:2]
 
@@ -55,6 +52,7 @@ if _py_version == (2, 7) and not IRONPYTHON:
     # read method of file-like returned by sock.makefile() is very slow.
     # So we copy io-based one from Python 3.
     from ._socketio import SocketIO
+
     def _makefile(sock, mode):
         return io.BufferedReader(SocketIO(sock, mode))
 elif _py_version == (2, 6):
@@ -63,6 +61,7 @@ elif _py_version == (2, 6):
     class SockFile(object):
         def __init__(self, sock):
             self._sock = sock
+
         def read(self, n):
             read = self._sock.recv(n)
             if len(read) == n:
@@ -92,7 +91,8 @@ TEXT_TYPES = set([
     FIELD_TYPE.STRING,
     FIELD_TYPE.TINY_BLOB,
     FIELD_TYPE.VAR_STRING,
-    FIELD_TYPE.VARCHAR])
+    FIELD_TYPE.VARCHAR,
+    FIELD_TYPE.GEOMETRY])
 
 sha_new = partial(hashlib.new, 'sha1')
 
@@ -107,7 +107,7 @@ DEFAULT_CHARSET = 'latin1'
 MAX_PACKET_LEN = 2**24-1
 
 
-def dump_packet(data):
+def dump_packet(data): # pragma: no cover
     def is_ascii(data):
         if 65 <= byte2int(data) <= 122:
             if isinstance(data, int):
@@ -137,7 +137,7 @@ def dump_packet(data):
 def _scramble(password, message):
     if not password:
         return b'\0'
-    if DEBUG: print('password=' + password)
+    if DEBUG: print('password=' + str(password))
     stage1 = sha_new(password).digest()
     stage2 = sha_new(stage1).digest()
     s = sha_new()
@@ -197,14 +197,12 @@ def _hash_password_323(password):
     nr2 = 0x12345671
 
     for c in [byte2int(x) for x in password if x not in (' ', '\t')]:
-        nr^= (((nr & 63)+add)*c)+ (nr << 8) & 0xFFFFFFFF
-        nr2= (nr2 + ((nr2 << 8) ^ nr)) & 0xFFFFFFFF
-        add= (add + c) & 0xFFFFFFFF
+        nr ^= (((nr & 63) + add) * c) + (nr << 8) & 0xFFFFFFFF
+        nr2 = (nr2 + ((nr2 << 8) ^ nr)) & 0xFFFFFFFF
+        add = (add + c) & 0xFFFFFFFF
 
-    r1 = nr & ((1 << 31) - 1) # kill sign bits
+    r1 = nr & ((1 << 31) - 1)  # kill sign bits
     r2 = nr2 & ((1 << 31) - 1)
-
-    # pack
     return struct.pack(">LL", r1, r2)
 
 
@@ -507,8 +505,9 @@ class Connection(object):
                  read_default_file=None, conv=decoders, use_unicode=None,
                  client_flag=0, cursorclass=Cursor, init_command=None,
                  connect_timeout=None, ssl=None, read_default_group=None,
-                 compress=None, named_pipe=None, no_delay=False,
-                 autocommit=False, db=None, passwd=None, local_infile=False):
+                 compress=None, named_pipe=None, no_delay=None,
+                 autocommit=False, db=None, passwd=None, local_infile=False,
+                 max_allowed_packet=16*1024*1024, defer_connect=False):
         """
         Establish a connection to the MySQL database. Accepts several
         arguments:
@@ -539,13 +538,21 @@ class Connection(object):
         read_default_group: Group to read from in the configuration file.
         compress; Not supported
         named_pipe: Not supported
-        no_delay: Disable Nagle's algorithm on the socket
+        no_delay: Disable Nagle's algorithm on the socket. (deprecated, default: True)
         autocommit: Autocommit mode. None means use server default. (default: False)
         local_infile: Boolean to enable the use of LOAD DATA LOCAL command. (default: False)
+        max_allowed_packet: Max size of packet sent to server in bytes. (default: 16MB)
+        defer_connect: Don't explicitly connect on contruction - wait for connect call.
+            (default: False)
 
         db: Alias for database. (for compatibility to MySQLdb)
         passwd: Alias for password. (for compatibility to MySQLdb)
         """
+        if no_delay is not None:
+            warnings.warn("no_delay option is deprecated", DeprecationWarning)
+            no_delay = bool(no_delay)
+        else:
+            no_delay = True
 
         if use_unicode is None and sys.version_info[0] > 2:
             use_unicode = True
@@ -586,14 +593,16 @@ class Connection(object):
             if not read_default_group:
                 read_default_group = "client"
 
-            cfg = configparser.RawConfigParser()
+            cfg = Parser()
             cfg.read(os.path.expanduser(read_default_file))
 
-            def _config(key, default):
+            def _config(key, arg):
+                if arg:
+                    return arg
                 try:
                     return cfg.get(read_default_group, key)
                 except Exception:
-                    return default
+                    return arg
 
             user = _config("user", user)
             password = _config("password", password)
@@ -641,10 +650,14 @@ class Connection(object):
         self.decoders = conv
         self.sql_mode = sql_mode
         self.init_command = init_command
-        self._connect()
+        self.max_allowed_packet = max_allowed_packet
+        if defer_connect:
+            self.socket = None
+        else:
+            self.connect()
 
     def close(self):
-        ''' Send the quit message and close the socket '''
+        """Send the quit message and close the socket"""
         if self.socket is None:
             raise err.Error("Already closed")
         send_data = struct.pack('<iB', 1, COMMAND.COM_QUIT)
@@ -690,7 +703,7 @@ class Connection(object):
         return ok
 
     def _send_autocommit_mode(self):
-        ''' Set whether or not to commit after every execute() '''
+        """Set whether or not to commit after every execute()"""
         self._execute_command(COMMAND.COM_QUERY, "SET AUTOCOMMIT = %s" %
                               self.escape(self.autocommit_mode))
         self._read_ok_packet()
@@ -701,12 +714,12 @@ class Connection(object):
         self._read_ok_packet()
 
     def commit(self):
-        ''' Commit changes to stable storage '''
+        """Commit changes to stable storage"""
         self._execute_command(COMMAND.COM_QUERY, "COMMIT")
         self._read_ok_packet()
 
     def rollback(self):
-        ''' Roll back the current transaction '''
+        """Roll back the current transaction"""
         self._execute_command(COMMAND.COM_QUERY, "ROLLBACK")
         self._read_ok_packet()
 
@@ -723,7 +736,7 @@ class Connection(object):
         self._read_ok_packet()
 
     def escape(self, obj, mapping=None):
-        ''' Escape whatever value you pass to it  '''
+        """Escape whatever value you pass to it"""
         if isinstance(obj, str_type):
             return "'" + self.escape_string(obj) + "'"
         return escape_item(obj, self.charset, mapping=mapping)
@@ -739,17 +752,17 @@ class Connection(object):
         return escape_string(s)
 
     def cursor(self, cursor=None):
-        ''' Create a new cursor to execute queries with '''
+        """Create a new cursor to execute queries with"""
         if cursor:
             return cursor(self)
         return self.cursorclass(self)
 
     def __enter__(self):
-        ''' Context manager that returns a Cursor '''
+        """Context manager that returns a Cursor"""
         return self.cursor()
 
     def __exit__(self, exc, value, traceback):
-        ''' On successful exit, commit. On exception, rollback. '''
+        """On successful exit, commit. On exception, rollback"""
         if exc:
             self.rollback()
         else:
@@ -757,10 +770,13 @@ class Connection(object):
 
     # The following methods are INTERNAL USE ONLY (called from Cursor)
     def query(self, sql, unbuffered=False):
-        #if DEBUG:
-        #    print("DEBUG: sending query:", sql)
+        # if DEBUG:
+        #     print("DEBUG: sending query:", sql)
         if isinstance(sql, text_type) and not (JYTHON or IRONPYTHON):
-            sql = sql.encode(self.encoding)
+            if PY2:
+                sql = sql.encode(self.encoding)
+            else:
+                sql = sql.encode(self.encoding, 'surrogateescape')
         self._execute_command(COMMAND.COM_QUERY, sql)
         self._affected_rows = self._read_query_result(unbuffered=unbuffered)
         return self._affected_rows
@@ -778,10 +794,10 @@ class Connection(object):
         return self._read_ok_packet()
 
     def ping(self, reconnect=True):
-        ''' Check if the server is alive '''
+        """Check if the server is alive"""
         if self.socket is None:
             if reconnect:
-                self._connect()
+                self.connect()
                 reconnect = False
             else:
                 raise err.Error("Already closed")
@@ -790,7 +806,7 @@ class Connection(object):
             return self._read_ok_packet()
         except Exception:
             if reconnect:
-                self._connect()
+                self.connect()
                 return self.ping(False)
             else:
                 raise
@@ -804,31 +820,31 @@ class Connection(object):
         self.charset = charset
         self.encoding = encoding
 
-    def _connect(self):
-        sock = None
+    def connect(self, sock=None):
         try:
-            if self.unix_socket and self.host in ('localhost', '127.0.0.1'):
-                sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
-                sock.settimeout(self.connect_timeout)
-                sock.connect(self.unix_socket)
-                self.host_info = "Localhost via UNIX socket"
-                if DEBUG: print('connected using unix_socket')
-            else:
-                while True:
-                    try:
-                        sock = socket.create_connection(
-                            (self.host, self.port), self.connect_timeout)
-                        break
-                    except (OSError, IOError) as e:
-                        if e.errno == errno.EINTR:
-                            continue
-                        raise
-                self.host_info = "socket %s:%d" % (self.host, self.port)
-                if DEBUG: print('connected using socket')
-            sock.settimeout(None)
-            sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
-            if self.no_delay:
-                sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+            if sock is None:
+                if self.unix_socket and self.host in ('localhost', '127.0.0.1'):
+                    sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+                    sock.settimeout(self.connect_timeout)
+                    sock.connect(self.unix_socket)
+                    self.host_info = "Localhost via UNIX socket"
+                    if DEBUG: print('connected using unix_socket')
+                else:
+                    while True:
+                        try:
+                            sock = socket.create_connection(
+                                (self.host, self.port), self.connect_timeout)
+                            break
+                        except (OSError, IOError) as e:
+                            if e.errno == errno.EINTR:
+                                continue
+                            raise
+                    self.host_info = "socket %s:%d" % (self.host, self.port)
+                    if DEBUG: print('connected using socket')
+                    if self.no_delay:
+                        sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+                sock.settimeout(None)
+                sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
             self.socket = sock
             self._rfile = _makefile(sock, 'rb')
             self._get_server_information()
@@ -948,31 +964,31 @@ class Connection(object):
         if isinstance(sql, text_type):
             sql = sql.encode(self.encoding)
 
-        chunk_size = min(MAX_PACKET_LEN, len(sql) + 1)  # +1 is for command
+        chunk_size = min(self.max_allowed_packet, len(sql) + 1)  # +1 is for command
 
         prelude = struct.pack('<iB', chunk_size, command)
         self._write_bytes(prelude + sql[:chunk_size-1])
         if DEBUG: dump_packet(prelude + sql)
 
-        if chunk_size < MAX_PACKET_LEN:
+        if chunk_size < self.max_allowed_packet:
             return
 
         seq_id = 1
         sql = sql[chunk_size-1:]
         while True:
-            chunk_size = min(MAX_PACKET_LEN, len(sql))
+            chunk_size = min(self.max_allowed_packet, len(sql))
             prelude = struct.pack('<i', chunk_size)[:3]
             data = prelude + int2byte(seq_id%256) + sql[:chunk_size]
             self._write_bytes(data)
             if DEBUG: dump_packet(data)
             sql = sql[chunk_size:]
-            if not sql and chunk_size < MAX_PACKET_LEN:
+            if not sql and chunk_size < self.max_allowed_packet:
                 break
             seq_id += 1
 
     def _request_authentication(self):
         self.client_flag |= CLIENT.CAPABILITIES
-        if self.server_version.startswith('5'):
+        if int(self.server_version.split('.', 1)[0]) >= 5:
             self.client_flag |= CLIENT.MULTI_RESULTS
 
         if self.user is None:
@@ -991,12 +1007,13 @@ class Connection(object):
             next_packet += 1
 
             if DEBUG: dump_packet(data)
-
             self._write_bytes(data)
+
+            cert_reqs = ssl.CERT_NONE if self.ca is None else ssl.CERT_REQUIRED
             self.socket = ssl.wrap_socket(self.socket, keyfile=self.key,
                                           certfile=self.cert,
                                           ssl_version=ssl.PROTOCOL_TLSv1,
-                                          cert_reqs=ssl.CERT_REQUIRED,
+                                          cert_reqs=cert_reqs,
                                           ca_certs=self.ca)
             self._rfile = _makefile(self.socket, 'rb')
 
@@ -1012,7 +1029,6 @@ class Connection(object):
         next_packet += 2
 
         if DEBUG: dump_packet(data)
-
         self._write_bytes(data)
 
         auth_packet = self._read_packet()
@@ -1281,10 +1297,8 @@ class LoadLocalFile(object):
         seq_id = 2
         try:
             with open(self.filename, 'rb') as open_file:
-                chunk_size = MAX_PACKET_LEN
-                prelude = b""
+                chunk_size = self.connection.max_allowed_packet
                 packet = b""
-                packet_size = 0
 
                 while True:
                     chunk = open_file.read(chunk_size)
@@ -1301,5 +1315,3 @@ class LoadLocalFile(object):
             # send the empty packet to signify we are done sending data
             packet = struct.pack('<i', 0)[:3] + int2byte(seq_id)
             self.connection._write_bytes(packet)
-
-# g:khuno_ignore='E226,E301,E701'
